<?php
/*
Plugin Name: SyntaxHub-SecureLog
Plugin URI: https://syn-c.jp/service/syntax-web-master/syntaxhub-securelog/
Description: WordPressの管理画面の不正ログイン試行を記録し、一定回数の失敗でIPをブロック
Version: 1.0.3
Author: SytaxCloud LLC
Author URI: https://syn-c.jp/
License: GPL2
License URI: https://www.gnu.org/licenses/gpl-2.0.html
Text Domain: SyntaxHub-SecureLog
*/

// プラグインの有効化時にテーブルを作成
register_activation_hook(__FILE__, 'create_plugin_tables');

function create_plugin_tables() {
    global $wpdb;
    $charset_collate = $wpdb->get_charset_collate();

    $sql1 = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}login_errors (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        username varchar(60) NOT NULL,
        ip_address varchar(45) NOT NULL,
        os varchar(255) NOT NULL,
        date datetime NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";

    $sql2 = "CREATE TABLE IF NOT EXISTS {$wpdb->prefix}protected_ips (
        id mediumint(9) NOT NULL AUTO_INCREMENT,
        ip_address varchar(45) NOT NULL,
        blocked_until datetime NOT NULL,
        PRIMARY KEY  (id)
    ) $charset_collate;";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
    dbDelta($sql1);
    dbDelta($sql2);

    // デフォルト設定の追加
    add_option('limit_time', 5);
    add_option('limit_count', 3);
    add_option('block_duration', 60);
    add_option('block_redirect_url', '');
}

// ログデータの収集と保存
function collect_login_error_data($username) {
    global $wpdb;
    $ip_address = $_SERVER['REMOTE_ADDR'];
    $os = $_SERVER['HTTP_USER_AGENT'];
    
    $wpdb->insert(
        $wpdb->prefix . 'login_errors',
        array(
            'username' => $username,
            'ip_address' => $ip_address,
            'os' => $os,
            'date' => current_time('mysql')
        )
    );

    limit_login_attempts($ip_address);
}
add_action('wp_login_failed', 'collect_login_error_data');

// IPアドレスによるログイン試行制限
function limit_login_attempts($ip_address) {
    global $wpdb;
    $limit_time = get_option('limit_time', 5);
    $limit_count = get_option('limit_count', 3);
    $block_duration = get_option('block_duration', 60);
    
    $count = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$wpdb->prefix}login_errors 
        WHERE ip_address = %s AND date > DATE_SUB(NOW(), INTERVAL %d MINUTE)",
        $ip_address, $limit_time
    ));

    if ($count >= $limit_count) {
        $existing_block = $wpdb->get_row($wpdb->prepare(
            "SELECT * FROM {$wpdb->prefix}protected_ips WHERE ip_address = %s",
            $ip_address
        ));

        $blocked_until = date('Y-m-d H:i:s', time() + ($block_duration * 60));

        if ($existing_block) {
            $wpdb->update(
                $wpdb->prefix . 'protected_ips',
                array('blocked_until' => $blocked_until),
                array('ip_address' => $ip_address)
            );
        } else {
            $wpdb->insert(
                $wpdb->prefix . 'protected_ips',
                array(
                    'ip_address' => $ip_address,
                    'blocked_until' => $blocked_until
                )
            );
        }
    }
}
// ログイン画面表示前のチェック
function check_ip_before_login() {
    $ip_address = $_SERVER['REMOTE_ADDR'];
    global $wpdb;

    $limit_time = get_option('limit_time', 5);
    $limit_count = get_option('limit_count', 3);

    $count = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$wpdb->prefix}login_errors 
        WHERE ip_address = %s AND date > DATE_SUB(NOW(), INTERVAL %d MINUTE)",
        $ip_address, $limit_time
    ));

    $is_blocked = $wpdb->get_var($wpdb->prepare(
        "SELECT COUNT(*) FROM {$wpdb->prefix}protected_ips 
        WHERE ip_address = %s AND blocked_until > NOW()",
        $ip_address
    ));

    if ($count >= $limit_count || $is_blocked) {
        $redirect_url = get_option('block_redirect_url', '');
        if (!empty($redirect_url)) {
            wp_redirect($redirect_url);
            exit;
        } else {
            wp_die('アクセスが制限されています。しばらく時間をおいてから再度お試しください。', 'アクセス制限', array('response' => 403));
        }
    }
}
add_action('login_init', 'check_ip_before_login');

// 管理メニューを追加
function add_admin_menu() {
    add_menu_page(
        'SyntaxHub-SecureLog', // ページタイトル
        'SecureLog', // メニュータイトル
        'manage_options', // 必要な権限
        'syntaxhub-securelog', // メニューのスラッグ
        'display_plugin_admin_page', // 表示用の関数
        'dashicons-shield', // アイコン
        80 // メニューの位置
    );
}
add_action('admin_menu', 'add_admin_menu');

// 管理ページの表示
function display_plugin_admin_page() {
    ?>
    <div class="wrap">
        <h1><?php echo esc_html(get_admin_page_title()); ?></h1>
        <form method="post" action="options.php">
            <?php
            settings_fields('syntaxhub_securelog_options');
            do_settings_sections('syntaxhub-securelog');
            submit_button('設定を保存');
            ?>
        </form>
        <?php
        unblock_ip_address();
        display_protected_ips();
        display_login_error_data();
        ?>
    </div>
    <?php
}

// 設定を登録
function syntaxhub_securelog_register_settings() {
    register_setting('syntaxhub_securelog_options', 'limit_time');
    register_setting('syntaxhub_securelog_options', 'limit_count');
    register_setting('syntaxhub_securelog_options', 'block_duration');
    register_setting('syntaxhub_securelog_options', 'block_redirect_url');

    add_settings_section(
        'syntaxhub_securelog_main',
        'メイン設定',
        'syntaxhub_securelog_section_text',
        'syntaxhub-securelog'
    );

    add_settings_field(
        'limit_time',
        'ログイン試行制限時間間隔（分）',
        'limit_time_callback',
        'syntaxhub-securelog',
        'syntaxhub_securelog_main'
    );

    add_settings_field(
        'limit_count',
        'ログイン試行制限回数',
        'limit_count_callback',
        'syntaxhub-securelog',
        'syntaxhub_securelog_main'
    );

    add_settings_field(
        'block_duration',
        'ブロック時間（分）',
        'block_duration_callback',
        'syntaxhub-securelog',
        'syntaxhub_securelog_main'
    );

    add_settings_field(
        'block_redirect_url',
        'ブロック時のリダイレクトURL',
        'block_redirect_url_callback',
        'syntaxhub-securelog',
        'syntaxhub_securelog_main'
    );
}
add_action('admin_init', 'syntaxhub_securelog_register_settings');

function syntaxhub_securelog_section_text() {
    echo '<p>SyntaxHub-SecureLogの主要設定です。</p>';
}

function limit_time_callback() {
    $value = get_option('limit_time', 5);
    echo '<input type="number" id="limit_time" name="limit_time" value="' . esc_attr($value) . '" />';
}

function limit_count_callback() {
    $value = get_option('limit_count', 3);
    echo '<input type="number" id="limit_count" name="limit_count" value="' . esc_attr($value) . '" />';
}

function block_duration_callback() {
    $value = get_option('block_duration', 60);
    echo '<input type="number" id="block_duration" name="block_duration" value="' . esc_attr($value) . '" />';
}

function block_redirect_url_callback() {
    $value = get_option('block_redirect_url', '');
    echo '<input type="text" id="block_redirect_url" name="block_redirect_url" value="' . esc_attr($value) . '" style="width: 100%;" />';
}

// 管理画面でのプロテクト解除機能
function unblock_ip_address() {
    if (isset($_POST['unblock_ip'])) {
        global $wpdb;
        $ip_to_unblock = sanitize_text_field($_POST['unblock_ip']);
        $wpdb->delete($wpdb->prefix . 'protected_ips', array('ip_address' => $ip_to_unblock));
        echo "<div class='updated'><p>IPアドレス {$ip_to_unblock} のブロックを解除しました。</p></div>";
    }
}

// 管理画面でのプロテクトIPアドレス表示
function display_protected_ips() {
    global $wpdb;
    $protected_ips = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}protected_ips");
    
    echo '<h3>プロテクトされているIPアドレス</h3>';
    
    if (empty($protected_ips)) {
        echo '<p>現在、プロテクトされているIPアドレスはありません。</p>';
    } else {
        echo '<table class="widefat">
            <thead>
                <tr>
                    <th>IPアドレス</th>
                    <th>ブロック解除時間</th>
                    <th>状態</th>
                    <th>操作</th>
                </tr>
            </thead>
            <tbody>';
        foreach ($protected_ips as $ip) {
            $is_active = strtotime($ip->blocked_until) > time();
            echo '<tr>
                <td>' . esc_html($ip->ip_address) . '</td>
                <td>' . esc_html($ip->blocked_until) . '</td>
                <td>' . ($is_active ? '有効' : '期限切れ') . '</td>
                <td>
                    <form method="post">
                        <input type="hidden" name="unblock_ip" value="' . esc_attr($ip->ip_address) . '">
                        <input type="submit" class="button" value="ブロック解除">
                    </form>
                </td>
            </tr>';
        }
         echo '</tbody></table>';
    }
    /*
    // デバッグ情報
    echo '<h4>デバッグ情報</h4>';
    echo '<pre>';
    $all_ips = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}protected_ips");
    print_r($all_ips);
    echo '</pre>';
    */
}  

// ログイン失敗記録の削除処理
function delete_login_error_records() {
    if (isset($_POST['delete_records']) && isset($_POST['record_ids'])) {
        global $wpdb;
        $ids = array_map('intval', $_POST['record_ids']);
        $ids_string = implode(',', $ids);
        
        $wpdb->query("DELETE FROM {$wpdb->prefix}login_errors WHERE id IN ($ids_string)");
        
        echo '<div class="updated"><p>選択したログを削除しました。</p></div>';
    }
}

// ログデータの表示
function display_login_error_data() {
    global $wpdb;
    $errors = $wpdb->get_results("SELECT * FROM {$wpdb->prefix}login_errors ORDER BY date DESC LIMIT 100");
    
    // Delete処理の実行
    delete_login_error_records();
    ?>
    <h3>ログイン失敗記録</h3>
    <form method="post" id="login-errors-form">
        <div style="margin: 10px 0;">
            <input type="submit" name="delete_records" class="button action" value="選択したログを削除" onclick="return confirm('選択したログを削除してもよろしいですか？');">
        </div>
        <table class="widefat">
            <thead>
                <tr>
                    <th width="30"><input type="checkbox" id="select-all"></th>
                    <th>日付</th>
                    <th>ユーザー名</th>
                    <th>IPアドレス</th>
                    <th>OS</th>
                </tr>
            </thead>
            <tbody>
                <?php foreach ($errors as $error): ?>
                <tr>
                    <td><input type="checkbox" name="record_ids[]" value="<?php echo esc_attr($error->id); ?>" class="record-checkbox"></td>
                    <td><?php echo esc_html($error->date); ?></td>
                    <td><?php echo esc_html($error->username); ?></td>
                    <td><?php echo esc_html($error->ip_address); ?></td>
                    <td><?php echo esc_html($error->os); ?></td>
                </tr>
                <?php endforeach; ?>
            </tbody>
        </table>
    </form>
    <script type="text/javascript">
    jQuery(document).ready(function($) {
        // 全選択/解除の処理
        $('#select-all').change(function() {
            $('.record-checkbox').prop('checked', $(this).prop('checked'));
        });
        
        // 個別のチェックボックスの状態変更時の処理
        $('.record-checkbox').change(function() {
            if ($('.record-checkbox:checked').length === $('.record-checkbox').length) {
                $('#select-all').prop('checked', true);
            } else {
                $('#select-all').prop('checked', false);
            }
        });
        
        // フォーム送信時のバリデーション
        $('#login-errors-form').submit(function(e) {
            if (!$('.record-checkbox:checked').length) {
                alert('削除するログを選択してください。');
                e.preventDefault();
            }
        });
    });
    </script>
    <?php
}